/*
 * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* $Id: xfs_common.h,v 1.26 2000/10/18 01:04:43 lha Exp $ */

#ifndef _xfs_common_h
#define _xfs_common_h

#define XFS_MEM_NONE		0
#define XFS_MEM_MSGBUF		1
#define XFS_MEM_SENDRPC		2
#define XFS_MEM_DENTRY		3
#define XFS_MEM_READDIR		4
#define XFS_MEM_FOLLOWLINK	5
#define XFS_MEM_XNODE		6

void *xfs_alloc (u_int size, unsigned int service);
void xfs_free (void *, unsigned int service);
void xfs_tell_alloc(void);
void xfs_print_sleep_queue(void);

extern struct inode_operations xfs_file_inode_operations,
                               xfs_dir_inode_operations,
                               xfs_link_inode_operations,
                               xfs_dead_inode_operations;

extern struct file_operations xfs_file_operations,
			      xfs_dead_operations,
			      xfs_dir_operations,
			      xfs_link_operations;

struct xfs_dentry_data {
    int valid;
    struct semaphore sem;
};

#define DENTRY_TO_XDENTRY(d) ((struct xfs_dentry_data *)((d)->d_fsdata))

int
xfs_d_init (struct dentry *dentry);

#define DENTRY_TO_INODE(x) ((x)->d_inode)

void print_aliases(struct inode *inode);
void print_childs(struct dentry *dentry);

size_t
strlcpy (char *dst, const char *src, size_t dst_sz);

#endif /* _xfs_common_h */
